from zExceptions import Unauthorized
if REQUEST is not None:
  raise Unauthorized

from DateTime import DateTime

portal = context.getPortalObject()
instance_tree = context

tag = '%s_%s' % (instance_tree.getUid(), script.id)
activate_kw = {'tag': tag}
if portal.portal_activities.countMessageWithTag(tag) > 0:
  # nothing to do
  return


def storeWorkflowComment(document, comment):
  portal.portal_workflow.doActionFor(document, 'edit_action', comment=comment)


def newOpenOrder(start_date):
  new_open_sale_order = portal.open_sale_order_module.newContent(portal_type="Open Sale Order")
  new_open_sale_order.edit(
    specialise=specialise,
    effective_date=DateTime(),
    start_date=start_date,
    # Ensure stop date value is higher than start date
    # it will be updated by OpenSaleOrder_updatePeriod
    stop_date=start_date + 1,
    activate_kw=activate_kw,
    destination_value=person,
    destination_section_value=person,
    destination_decision_value=person,
    title="%s SlapOS Subscription" % person.getTitle(),
    ledger_value=portal.portal_categories.ledger.automated,
    destination_project_value=instance_tree.getFollowUpValue(),
  )

  new_open_sale_order.order(activate_kw=activate_kw)
  new_open_sale_order.validate(activate_kw=activate_kw)
  return new_open_sale_order

if instance_tree.getCausalityState() == 'diverged':
  open_order = None
  person = instance_tree.getDestinationSectionValue(portal_type="Person")
  # Template document does not have person relation
  if person is not None:

    # Search an existing related open order
    open_order_line = portal.portal_catalog.getResultValue(
      portal_type='Open Sale Order Line',
      default_aggregate_uid=instance_tree.getUid())

    is_open_order_creation_needed = False

    # Simply check that it has never been simulated
    if instance_tree.getSlapState() == 'destroy_requested':
      # Line should be deleted
      if (open_order_line is not None) and (open_order_line.getValidationState() == "invalidated"):
        instance_tree.converge(comment="Last open order: %s" % open_order_line.getRelativeUrl())
      elif open_order_line is None:
        # User has no Open Sale Order (likely).
        # No need to charge, as it was never allocated
        is_open_order_creation_needed = False
        instance_tree.converge(comment="No open order needed as it was never allocated")

    elif open_order_line is None:
      # Let's add
      is_open_order_creation_needed = True

    # Let's create the open order
    if is_open_order_creation_needed:

      hosting_subscription = portal.hosting_subscription_module.newContent(
        portal_type="Hosting Subscription",
        title=instance_tree.getTitle(),
        ledger_value=portal.portal_categories.ledger.automated,
      )
      hosting_subscription.validate()
      start_date = hosting_subscription.HostingSubscription_calculateSubscriptionStartDate()

      open_sale_order = newOpenOrder(start_date)
      open_order_explanation = ""
      # Add lines
      open_order_line = open_sale_order.newContent(portal_type="Open Sale Order Line")

      # Search for matching resource
      service = instance_tree.InstanceTree_getSoftwareProduct()
      if (service is None) or (service.getUse() != 'trade/sale'):
        raise NotImplementedError('No Software Product defined for %s' % instance_tree.getRelativeUrl())

      #assert instance_tree.getPortalType() in service.getRequiredAggregatedPortalTypeList()
      software_release = portal.portal_catalog.getResultValue(
        parent_uid=service.getUid(),
        url_string=instance_tree.getUrlString(),
        portal_type="Software Product Release Variation"
      )
      software_type = portal.portal_catalog.getResultValue(
        parent_uid=service.getUid(),
        title=instance_tree.getSourceReference(),
        portal_type="Software Product Type Variation"
      )

      resource_vcl = [
        'software_release/%s' % software_release.getRelativeUrl(),
        'software_type/%s' % software_type.getRelativeUrl()
      ]
      resource_vcl.sort()
      assert len(resource_vcl) == 2, service

      edit_kw = {
        'resource_value': service,
        'variation_category_list': resource_vcl,
        'quantity_unit': service.getQuantityUnit(),
        'base_contribution_list': service.getBaseContributionList(),
        'use': service.getUse()
      }

      subscription_request = instance_tree.getAggregateRelatedValue(portal_type="Subscription Request")
      # Define the start date of the period, this can variates with the time.
      # start_date_delta = 0
      if subscription_request is not None:
        # Quantity is double because the first invoice has to
        # charge for 2 months
        edit_kw['quantity'] = subscription_request.getQuantity()

      open_order_line.edit(
        activate_kw=activate_kw,
        # stop_date=calculateOpenOrderLineStopDate(open_sale_order_line,
        #         instance_tree, start_date_delta=start_date_delta),
        **edit_kw
      )

      base_id = 'path'
      # XXX XXX XXX TODO WRONG: it should not get the first random variations
      # but match the instance tree parameters instead
      cell_key = list(open_order_line.getCellKeyList(base_id=base_id))[0]
      open_order_cell = open_order_line.newCell(
        base_id=base_id,
        portal_type='Open Sale Order Cell',
        *cell_key
      )
      open_order_cell.edit(
        mapped_value_property_list=['price','quantity'],
        quantity=1,
        predicate_category_list=cell_key,
        variation_category_list=cell_key,
        aggregate_value=[
          hosting_subscription,
          instance_tree
        ],
      )

      open_order = open_order_line.getParentValue()
      open_order.SaleOrder_applySaleTradeCondition(batch_mode=1, force=1)

      # Check compatibility with previous template
      if specialise is not None:
        assert open_order.getSpecialise() == specialise

      open_order_cell.edit(
        price=open_order_cell.getPrice()
      )
      # XXX XXX XXX assert open_order_cell.getPrice() is not None

      storeWorkflowComment(open_order_line, "Created for %s" % instance_tree.getRelativeUrl())
      # instance_tree.converge(comment="Last open order: %s" % open_sale_order_line.getRelativeUrl())
      open_order_explanation = "Added %s." % str(open_order_line.getId())

      storeWorkflowComment(open_sale_order, open_order_explanation)

    if open_order_line is not None:
      open_order = open_order_line.getParentValue()

      assert open_order_line.getResource().startswith('software_product')
      assert open_order_line.getQuantityUnit() == 'time/month', open_order_line.getQuantityUnit()
      assert open_order_line.getBaseContribution() == 'base_amount/invoicing/discounted'
      assert open_order_line.getBaseContributionList()[1] == 'base_amount/invoicing/taxable'
      assert open_order_line.getUse() == 'trade/sale'
      #assert open_order_line.getPrice() == 1, open_order_line.getPrice()
      #assert open_order_line.getQuantity() == 1

      open_order.OpenSaleOrder_updatePeriod()

    # Person_storeOpenSaleOrderJournal should fix all divergent Instance Tree in one run
    assert instance_tree.getCausalityState() == 'solved'
    return open_order
